******************************************************************************
*
*       Function:   EXPIRE_PROC
*       Usage:      called as an AST service routine
*       Purpose:    The procedure is the called by the expiration of a
*                   $SETIMR request.  The procedure deletes an idle process
*                   from the system.
*       Output:     A deleted process, and a TERMINATION message to the
*                   mailbox specified when this process was created.
*
******************************************************************************

        subroutine expire_proc (proc)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            record /sub_proc/ proc
            integer stat
            external signal, sys$cantim, LIB$_INTLOGERR
            integer sys$cantim

*           Cancel anyother pending timer requests (There should not be any)

            stat = sys$cantim (proc,)

*           Check to see that the process is not starting to be used.

            if (proc.avail) then

*               Check for program errors.

                if (proc.started) then

*                   Set the flag so that proc_deleted() and report_deleted()
*                   will know that we mean for the process to go away.

                    proc.dying = .true.

*                   Delete the process.

                    stat = sys$delprc (proc.pid,)
                    if ((stat.and.1).ne.1) then
                        call log_str (%val (stat), 'Can''t expire subprocess')
                    endif
                else
                    call log_str (%val (SS$_NONEXPR),
     .                                          'Process to expire is gone')
                endif
            else
                call log_str (LIB$_INTLOGERR,
     .                              'Race condition found in expire_proc()')
            endif
        end
