******************************************************************************
*
*       Function:   RESET_STREAM
*       Usage:      call reset_stream (SMB_stream_id, data_descr,
*                                                           SMB_request_id)
*       Purpose:    This routine is used to process the RESET_STREAM request
*                   from the JBC.  This routine DELETES all processes that
*                   are associated with this stream.  If this is the last
*                   stream to be stopped, then all idle subprocesses are
*                   also deleted.
*       Output:     The stream is stopped, the appropriate subprocesses are
*                   deleted, and possibly the image is $EXIT'd
*
******************************************************************************

        subroutine reset_stream (stream, data, request)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer stream, request, i, stat, sys$dassgn
            record /dynam/ data
            external log_str, sys$dassgn

*           Get only the stream specific info.

            last_stream = stream
            last_idx = MAXPROCS+1

            call get_info (stream, data, request, -1)

            num_streams = num_streams - 1

*           Delete all processes currently running under this stream

            do i=1, MAXPROCS
                if ((.not. procs(i).avail) .and.
     .                      (procs(i).started) .and.
     .                      (procs(i).stream .eq. stream)) then
                    last_idx = i
                    procs(i).dying = .true.
                    stat = sys$delprc (procs(i).pid,)
                    if ((stat.and.1).ne.1) then
                        call log_str (%val (stat),
     .                          'Can''t delete process in reset_stream')
                    endif
                endif
            end do

            call reply (SS$_NORMAL, SMBMSG$K_RESET_STREAM, stream)

*           Delete all remaining subprocesses that are not active
*           iff we are the last stream left.  NOTE that they will
*           die anyway when we $EXIT, but it is usually better to
*           clean up before exiting.

            if (num_streams .eq. 0) then

*               Delete any active processes.

                do i=1, MAXPROCS
                    if (procs(i).started) then
                        last_idx = i
                        procs(i).dying = .true.
                        stat = sys$delprc (procs(i).pid,)
                        if ((stat.and.1).ne.1) then
                            call log_str (%val (stat),
     .                          'Can''t delete process in reset_stream')
                        endif
                    endif
                end do

                call sys$exit (%val(SS$_NORMAL))
            endif
        end
