******************************************************************************
*
*       Function:   STOP_STREAM
*       Usage:      call stop_stream (SMB_stream_id, data_descr, SMB_request)
*       Purpose:    This routine is used to process the JBC's STOP_STREAM
*                   requests.  This routine is similar to the RESET_STREAM
*                   procedure.  It deletes the processes that are associated
*                   with the indicated stream, and then if it is the last
*                   stream to exit, it deletes any remaining idle processes,
*                   and then exits.
*       Output:     One less stream active in the symbiont.
*
******************************************************************************

        subroutine stop_stream (stream, data, request)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer stream, request, i, stat, sys$dassgn
            record /dynam/ data
            external get_info, reply, log_str, sys$dassgn

*           Get only the stream specific info.

            call get_info (stream, data, request, -1)

            last_stream = stream
            last_idx = MAXPROCS+1

            num_streams = num_streams - 1

*           Delete all processes currently running under this stream

            do i=1, MAXPROCS
                if ((.not. procs(i).avail) .and.
     .                      (procs(i).started) .and.
     .                      (procs(i).stream .eq. stream)) then
                    last_idx = i
                    procs(i).dying = .true.
                    stat = sys$delprc (procs(i).pid,)
                    if ((stat.and.1).ne.1) then
                        call log_str (%val (stat),
     .                          'Can''t delete process in stop_stream')
                    endif
                endif
            end do

            call reply (SS$_NORMAL, SMBMSG$K_STOP_STREAM, stream)

*           Delete all remaining subprocesses that are not active
*           iff we are the last stream left.  NOTE that they will
*           die anyway when we $EXIT, but it is usually better to
*           clean up before exiting.

            if (num_streams .eq. 0) then

*               Delete any active processes.

                do i=1, MAXPROCS
                    if (procs(i).started) then
                        last_idx = i
                        procs(i).dying = .true.
                        stat = sys$delprc (procs(i).pid,)
                        if ((stat.and.1).ne.1) then
                            call log_str (%val (stat),
     .                          'Can''t delete process in stop_stream')
                        endif
                    endif
                end do

                call sys$exit (%val(SS$_NORMAL))
            endif
        end
